//------------------------------------------------------------------------------
// Lamp : Open source game middleware
// Copyright (C) 2004  Junpei Ohtani ( Email : junpee@users.sourceforge.jp )
//
// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public
// License as published by the Free Software Foundation; either
// version 2.1 of the License, or (at your option) any later version.
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public
// License along with this library; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//------------------------------------------------------------------------------

/** @file
 * 実数矩形ヘッダ
 * @author Junpee
 */

#ifndef RECTANGLE_F_H_
#define RECTANGLE_F_H_

namespace Lamp{

class RectangleI;

//------------------------------------------------------------------------------
/**
 * 実数矩形
 *
 * このクラスは継承しないで下さい。
 */
class RectangleF{
public:
	//--------------------------------------------------------------------------
	// メンバ変数
	//--------------------------------------------------------------------------
	/// メンバ変数
	union{
		/// 各要素
		struct{
			/// X値
			float x;
			/// Y値
			float y;
			/// 幅
			float width;
			/// 高さ
			float height;
		};

		/// 配列
		float array[4];
	};

	//--------------------------------------------------------------------------
	// 定数
	//--------------------------------------------------------------------------
	/// ゼロ矩形
	static const RectangleF zero;

	/// 単位矩形
	static const RectangleF unit;

	//--------------------------------------------------------------------------
	// コンストラクタ
	//--------------------------------------------------------------------------
	/**
	 * コンストラクタ
	 *
	 * このコンストラクタは初期値の設定を行わないため値は不定です。
	 */
	RectangleF(){}

	/**
	 * コンストラクタ
	 * @param sourceX X値の初期値
	 * @param sourceY Y値の初期値
	 * @param sourceWidth 幅の初期値
	 * @param sourceHeight 高さの初期値
	 */
	RectangleF(float sourceX, float sourceY,
		float sourceWidth, float sourceHeight) :
		x(sourceX), y(sourceY), width(sourceWidth), height(sourceHeight){}

	/**
	 * コンストラクタ
	 * @param sourceArray 初期値配列
	 */
	explicit RectangleF(float sourceArray[4]) :
		x(sourceArray[0]), y(sourceArray[1]),
		width(sourceArray[2]), height(sourceArray[3]){}

	/**
	 * コンストラクタ
	 * @param source 設定する矩形
	 */
	explicit RectangleF(const RectangleI& source);

	//--------------------------------------------------------------------------
	// 値の設定
	//--------------------------------------------------------------------------
	/**
	 * 値の設定
	 * @param sourceX X値の設定値
	 * @param sourceY Y値の設定値
	 * @param sourceWidth 幅の設定値
	 * @param sourceHeight 高さの設定値
	 */
	inline void set(float sourceX, float sourceY,
		float sourceWidth, float sourceHeight){
		x = sourceX;
		y = sourceY;
		width = sourceWidth;
		height = sourceHeight;
	}

	/**
	 * 値の設定
	 * @param sourceArray 設定値配列
	 */
	inline void set(float sourceArray[4]){
		x = sourceArray[0];
		y = sourceArray[1];
		width = sourceArray[2];
		height = sourceArray[3];
	}

	/**
	 * 値の設定
	 * @param source 設定する矩形
	 */
	void set(const RectangleI& source);

	//--------------------------------------------------------------------------
	// 論理演算
	//--------------------------------------------------------------------------
	/**
	 * 同じ値かどうか
	 * @param target 比較する矩形
	 * @return 同じ値であればtrueを返す
	 */
	inline bool operator ==(const RectangleF& target) const{
		return ((x == target.x) && (y == target.y) &&
			(width == target.width) && (height == target.height));
	}

	/**
	 * 同じ値かどうか
	 * @param target 比較する矩形
	 * @param epsilon 誤差
	 * @return 誤差の範囲内で同じ値であればtrueを返す
	 */
	inline bool epsilonEquals(const RectangleF& target, float epsilon) const{
		Assert(epsilon >= 0.f);
		return (
			(Math::abs(x - target.x) <= epsilon) &&
			(Math::abs(y - target.y) <= epsilon) &&
			(Math::abs(width - target.width) <= epsilon) &&
			(Math::abs(height - target.height) <= epsilon));
	}

	/**
	 * 同じ値でないかどうか
	 * @param target 比較する矩形
	 * @return 同じ値でなければtrueを返す
	 */
	inline bool operator !=(const RectangleF& target) const{
		return ((x != target.x) || (y != target.y) ||
			(width != target.width) || (height != target.height));
	}

	/**
	 * 同じ値でないかどうか
	 * @param target 比較する矩形
	 * @param epsilon 誤差
	 * @return 誤差の範囲内で同じでない値であればtrueを返す
	 */
	inline bool notEpsilonEquals(
		const RectangleF& target, float epsilon) const{
		Assert(epsilon >= 0.f);
		return (
			(Math::abs(x - target.x) > epsilon) ||
			(Math::abs(y - target.y) > epsilon) ||
			(Math::abs(width - target.width) > epsilon) ||
			(Math::abs(height - target.height) > epsilon));
	}

	//--------------------------------------------------------------------------
	// その他
	//--------------------------------------------------------------------------
	/**
	 * 文字列化
	 * @return 矩形の文字列表記
	 */
	inline String toString() const{
		String returnString;
		returnString.format("( %.8f, %.8f, %.8f, %.8f )", x, y, width, height);
		return returnString;
	}

};

//------------------------------------------------------------------------------
} // End of namespace Lamp
#endif // End of RECTANGLE_F_H_
//------------------------------------------------------------------------------
